<?php
// $Id$
//
// 6ga-dlurl.php
//
//	Copyright (c) 2005,2013 Hiro Sugawara
//	Licensed under terms of GPL 2.0
//
//	Script to get an URL for the next downloadable file and
//	update a recorded file's status
//
//	Arguments:
//		If none, get information of the next downloadable file
//			tv-user:	user name for rsync
//			tv-dl-url:	URL for rsync
//			tv-xfrd-url:	URL for xfrd setting
//			
//		Arguments for updating xfrd byte count:
//		sched=<schedule file basename>
//			tv-file:	schedule file basename
//			tv-stat:	current recording status
//		xfrd=<size in kB>	If set, xfrd file is updated with the value
//
//		Argument for lower ranked downloadable file: (to be implemented)
//		rank=<n>
//			n = 1... n'th next downloadable
//
include('6ga-common.inc');

$tvuser	= 'tv';

$xfrd	= @$_GET['xfrd'];
$sched	= @$_GET['sched'];

$recdir	= "$VIDEOROOTDIR/recorded";

//
// Print iEPG contents
//
function printiEPG($fbname) {
	global $recdir;

	if (($fh = @fopen("$recdir/$fbname.iepg", 'r'))) {
		while (($line = fgets($fh)))
			print($line);
		fclose($fh);
	}
}

header('Content-Type: text/plain', TRUE);

//
// If xfrd action is given, set the value to xfrd file
// Otherwise if sched is given show file information
//
if ($sched) {
	if ("$xfrd") {
		umask(0133);
		if (($fh = fopen("$recdir/$sched.xfrd", 'w'))) {
			fputs($fh, "$xfrd\n");
			ftruncate($fh, ftell($fh));
			fclose($fh);
			if ($xfrd == 0)
				logit("$sched: started xfer");
			if ($xfrd < 0)
				logit("$sched: completed xfer");
		}
	} else {
		if (($fh = @fopen("$recdir/$sched.stat", 'r'))) {
			$stat = trim(fgets($fh, 50));
			fclose($fh);
		}
		echo <<<_END
tv-file: $sched
tv-stat: $stat

_END;
		printiEPG($sched);
	}
	exit(0);
}

//
// Filter to get ready recording
//
function readyOrXferring($fbname, $farg) {
	global $recdir;
	global $stat;

	if (($fh = @fopen("$recdir/$fbname.stat", 'r')) == NULL)
		return FALSE;
	$stat	= trim(fgets($fh));
	fclose($fh);
	if ($stat != 'Ready')
		return FALSE;
	if (($fh = @fopen("$recdir/$fbname.xfrd", 'r')) != NULL) {
		$xfrd	= trim(fgets($fh));
		fclose($fh);
		if ("$xfrd" && $xfrd < 0)
			return FALSE;
	}
	if (!aviFilePath("$recdir/$fbname") ||
		!file_exists("$recdir/$fbname.iepg"))
		return FALSE;

	return TRUE;
}

//
// Look for the best downloadable recorded file
//
if (($list = openList('recorded', 'readyOrXferring', '', 'stat', 'sort'))
		!= FALSE) {
	# Files are sorted by date/time. Oldest comes first.
	if (readList($list)) {
		$fbname	= $list['fbasename'];
		$avifile= aviFilePathList($list);
		$fsize10M	= (fsizeInKB($avifile) + 5000)/10000;
		$download	= $list;
		$datetime	= basename($fbname);

		date_default_timezone_set('JST');
		$datetime	= mktime(
						substr($datetime, 8, 2) + 0,
						substr($datetime,10, 2) + 0,
						substr($datetime,12, 2) + 0,
						substr($datetime, 4, 2) + 0,
						substr($datetime, 6, 2) + 0,
						substr($datetime, 0, 4) + 0);

		// Download the smallest file if the oldest is less than 7 days old.
		// Round file sizes by 10MB to pick up oldest among close-sized files.
		if ($datetime > time() - 7 * 24 * 60 * 60) {
			while (readList($list)) {
				$avifile= aviFilePathList($list);
				$thissize10M = (fsizeInKB($avifile) + 5000)/10000;
				if ($thissize10M > 0 && $thissize10M < $fsize10M) {
					$download	= $list;
					$fsize10M	= $thissize10M;
				}
			}
		}

		$myhost		= $_SERVER['SERVER_NAME'];
		$myport		= $_SERVER['SERVER_PORT'];
		$protocol	= $_SERVER['HTTPS'] ? 'https' : 'http';
		$fbname		= $download['fbasename'];
		$avifile	= aviFilePathList($download);
		$avifilebase= basename($avifile);
		$statfile	= preg_replace('/\..*$/', '.stat', $avifile);
		$myuri		= $_SERVER['REQUEST_URI'];
		$fsize		= fsizeInB($avifile);

		if (($fh = @fopen($statfile, 'r'))) {
			$stat = trim(fgets($fh, 50));
			fclose($fh);
		}

		echo <<<_END
tv-user: $tvuser
tv-dl-url: $tvuser@$myhost:$avifile
tv-xfrd-url: $protocol://$myhost:$myport$myuri?sched=$fbname&xfrd=
tv-stat: $stat
tv-file-name-base: $fbname
tv-file-name: $avifilebase
file-size: $fsize

_END;
		printiEPG($fbname);
	}
	closeList($list);
}
?>
