<?php
// $Id$
//
// 6ga-schedule.php
//
//	Copyright (c) 2005 Hiro Sugawara
//	Licensed under terms of GPL 2.0
//
//	Script to schedule a new recording
//
//	Arguments:
//		iepg_*=value
//
include('6ga-common.inc');

$NUMCAPCARDS	= $NUMVIDEODEVICES;

$DOWMAP	= array('Mon' => '1000000', 'Tue' => '0100000', 'Wed' => '0010000',
				'Thu' => '0001000', 'Fri' => '0000100', 'Sat' => '0000010',
				'Sun' => '0000001', 'M-F' => '1111100');
$DOWNUM	= array('Mon' => 0, 'Tue' => 1, 'Wed' => 2,
				'Thu' => 3, 'Fri' => 4, 'Sat' => 5, 'Sun' => 6);
$WEEKDAY= array('Mon' => 1, 'Tue' => 1, 'Wed' => 1,
				'Thu' => 1, 'Fri' => 1, 'Sat' => 0, 'Sun' => 0);
$DOW	= array('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun');


if (!$thistab) {
	$thistab	= $_COOKIE['tab'];
}
setcookie('tab', $thistab);
$bgcolor	= $TABCOLOR[$thistab];

saveiepg();
if (@$DOWMAP[$program]) {
	$iepg['every']	= $program;
}

$alerttext	= '';

//
// Return start and end minute-of-the-week's
//
function startEndMOW($iepg) {
	global $DOWNUM;

	$shm	= preg_split('/:/', $iepg['start']);
	$startm	= $DOWNUM[$iepg['every']] * 24 * 60 + $shm[0] * 60 + $shm[1];
	$endm	= $startm + programTime($iepg);
	return array('start' => $startm, 'end' => $endm);
}

$confloc	= '';

// Return UNIX time notation
function programStart($iepg) {
	$shm	= preg_split('/:/', $iepg['start']);
	return	mktime($shm[0], $shm[1], 0,
					$iepg['month'], $iepg['date'], $iepg['year']);
}

//
// Determine overlapping of start/end minute of the week
//
function semsOverlap($sem, $sem2) {
	global $confloc;

	//
	// Inclusive cases
	//
	if ($sem['start'] <= $sem2['start'] && $sem2['end'] <= $sem['end']) {
		$confloc	.= ' inclusive';
		return $sem2;
	}
	if ($sem2['start'] <= $sem['start'] && $sem['end'] <= $sem2['end']) {
		$confloc	.= ' inclusive';
		return $sem;
	}

	//
	// Staggering cases
	//
	if ($sem['start'] < $sem2['start'] && $sem2['start'] < $sem['end']) {
		$confloc	.= ' staggering';
		return array('start' => $sem2['start'], 'end' => $sem['end']);
	}
	if ($sem2['start'] < $sem['start'] && $sem['start'] < $sem2['end']) {
		$confloc	.= ' staggering';
		return array('start' => $sem['start'], 'end' => $sem2['end']);
	}

	//
	// Take care wrap-around cases
	//
	if ($sem['end'] >= 7 * 24 * 60
		&& $sem2['start'] < $sem['end'] - 7 * 24 * 60) {
		if ($sem2['end'] < $sem['end'])
			return $sem2;
		else
			return array('start' => $sem2['start'],
						 'end' => $sem['end'] - 7 * 24 * 60);
	}
	if ($sem2['end'] >= 7 * 24 * 60
		&& $sem['start'] < $sem2['end'] - 7 * 24 * 60) {
		if ($sem['end'] < $sem2['end'])
			return $sem;
		else
			return array('start' => $sem['start'],
						 'end' => $sem2['end'] - 7 * 24 * 60);
	}
	return NULL;
}

//
// Check conflict between two regular schedules
//
function conflictRR($iepg, &$iepgc, $numcapcards) {
	global $alerttext, $confloc;

	//
	// Check 1 dow regular schedule against exsisting regular schedules
	//
	function conflictR1R($iepg, &$iepgc, $numcapcards) {
		global $alerttext, $confloc;

		$conflicters	= array();

		$stime	= programStart($iepg);
		$etime	= $stime + programTime($iepg) * 60;

		$list = openList('regular');
		while (readList($list)) {
			$iepg2	= $list['iepg'];
			if ($iepg2['every'] == 'M-F') {
				if ($iepg['every'] == 'Sun' || $iepg['every'] == 'Sat') {
					continue;
				}
			} else if ($iepg['every'] != $iepg2['every']) {
				continue;
			}
			$stime2	= programStart($iepg2);
			$etime2	= $stime2 + programTime($iepg2) * 60;
			if (($stime <= $stime2 && $stime2 < $etime)
				|| ($stime2 <= $stime && $stime < $etime2)) {
				$conflicters[]	= $iepg2;
			}
		}
		closeList($list);

		if (count($conflicters) < $numcapcards) {
			return FALSE;
		}

		// Scan every minute to check conflicts with all potential conflicters
		for ($ctime = $stime; $ctime < $etime; $ctime += 60) {
			$result = 0;
			foreach ($conflicters as $iepgc) {
				if (programStart($iepgc) <= $ctime
					&& $ctime < programStart($iepgc) + programTime($iepgc) * 60) {
					if (++$result >= $numcapcards) {
						$confloc	.= sprintf(" %s %02:%02",
												$iepg['every'],
												$ctime/3600, ($ctime/60)%60);
						return TRUE;
					}
				}
			}
		}
		return FALSE;
	}

	//
	// Check duplication first
	//
	$result	= 0;
	$list = openList('regular');
	while (readList($list)) {
		if ($iepg['program-title'] == $list['iepg']['program-title']
			&& $iepg['start']	== $list['iepg']['start']
			&& $iepg['station']	== $list['iepg']['station']) {
			$confloc	= 'title/start/station match';
			if ($iepg['every']	== $list['iepg']['every']) {
				$result = 1;
				$confloc	= ' - SIMPLE DUPLICATE';
				break;
			}
			if ($iepg['every'] == 'M-F' && $WEEKDAY[$list['iepg']['every']]) {
				$result = 1;
				$confloc	= ' - M-F vs ' . $list['iepg']['every'];
				break;
			}
			if ($WEEKDAY[$iepg['every']] && $list['iepg']['every'] == 'M-F') {
				$result = 1;
				$confloc	= ' - ' . $list['iepg']['every'] . ' vs M-F';
				break;
			}
		}
	}
	closeList($list);
	if ($result) {
		$iepgc	= $list['iepg'];
		return TRUE;
	}

	//
	// M-F weekday program scans all weekdays and calls conflictRR recrusively
	//
	$result	= 0;
	if ($iepg['every'] == 'M-F') {
		foreach (array('Mon', 'Tue', 'Wed', 'Thu', 'Fri') as $wday) {
			$iepg['every'] = $wday;
			if (conflictR1R($iepg, $iepgc, $numcapcards)) {
				return TRUE;
			}
		}
	} else {
		return conflictR1R($iepg, $iepgc, $numcapcards);
	}
	return FALSE;
}

//
// Check conflict between a single schedule and a regular schedule
// ***FIXME*** capCards=1 only!!
//
function conflictSR($iepg, &$iepgc, $numcapcards) {
	global $alerttext, $confloc;

	function signatureMatch($fbn, $signature) {
		return preg_match("/-$signature\$/", $fbn);
	}

	// Set up temporary DOW for conflict detection only
	$iepg['every']	= getDOW($iepg);
	return conflictRR($iepg, $iepgc, $numcapcards);
}

//
// Check conflict between two single schedules
//
function conflictSS($iepg, &$iepgc, $numcapcards, $tab = 'scheduled') {
	global $alerttext, $confloc;

	//
	// Check duplication first
	//
	$result	= 0;
	$list = openList($tab);
	while (readList($list)) {
		if ($iepg['program-title'] == $list['iepg']['program-title']
			&& $iepg['start']	== $list['iepg']['start']
			&& $iepg['date']	== $list['iepg']['date']
			&& $iepg['month']	== $list['iepg']['month']
			&& $iepg['year']	== $list['iepg']['year']
			&& $iepg['station']	== $list['iepg']['station']) {
			$result = 1;
			break;
		}
	}
	closeList($list);
	if ($result) {
		$iepgc	= $list['iepg'];
		return TRUE;
	}

	$conflicters	= array();

	$stime	= programStart($iepg);
	$etime	= $stime + programTime($iepg) * 60;

	$list = openList('scheduled');
	while (readList($list)) {
		$iepg2	= $list['iepg'];
		$stime2	= programStart($iepg2);
		$etime2	= $stime2 + programTime($iepg2) * 60;
		if (($stime <= $stime2 && $stime2 < $etime)
			|| ($stime2 <= $stime && $stime < $etime2)) {
			$conflicters[]	= $iepg2;
		}
	}
	closeList($list);

	if (count($conflicters) < $numcapcards) {
		return FALSE;
	}

	// Scan every minute to check conflicts with all potential conflicters
	for ($ctime = $stime; $ctime < $etime; $ctime += 60) {
		$result = 0;
		foreach ($conflicters as $iepgc) {
			if (programStart($iepgc) <= $ctime
				&& $ctime < programStart($iepgc) + programTime($iepgc) * 60) {
				if (++$result >= $numcapcards) {
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

//
// Check schedule conflict
//
function conflict($iepg) {
	global $alerttext, $confloc;

	function conflictAny($iepg, &$iepgc) {
		global $NUMCAPCARDS;

		if (@$iepg['every']) {
			if (conflictRR($iepg, $iepgc, $NUMCAPCARDS))
				return TRUE;
			//if (conflictRS($iepg, $iepgc, $NUMCAPCARDS))
				//return TRUE;
		} else {
			if (conflictSS($iepg, $iepgc, $NUMCAPCARDS))
				return TRUE;
			if (conflictSR($iepg, $iepgc, $NUMCAPCARDS))
				return TRUE;
			if (conflictSS($iepg, $iepgc, $NUMCAPCARDS, 'recorded'))
				return TRUE;
		}
		return FALSE;
	}

	if (@!$iepg['channel']) {
		$alerttext	.= sprintf('Station \"%s\" unrecognized.\n',
								$iepg['station']);
		return TRUE;
	} elseif (conflictAny($iepg, $iepgc)) {
		$alerttext	.= sprintf('Recording of \"%s\"\n', $iepg['program-title']);
		if ($iepg['program-title'] == $iepgc['program-title']) {
			$alerttext .= 'is already scheduled.';
		} else {
			$alerttext .= sprintf('conflicts with \"%s\" [%s].',
									$iepgc['program-title'],
									@$iepgc['every']	? $iepgc['every']
													: $iepgc['start']);
		}
		if ($confloc)
			$alerttext	.= "\\n$confloc";
		return TRUE;
	}
	return FALSE;
}

/////////////////////////////////////////////////////////////////////
//
// Main code starts here
//
/////////////////////////////////////////////////////////////////////

if ($choice == 'schedule' && !conflict($iepg)) {
	$signature	= sprintf("%08x", time());
	$iepgtext	= '';
	foreach (array_keys($iepg) as $key) {
		// Store unescaped single quotes (apostrophes)
		$iepgtext .= sprintf("%s: %s\n", $key,
						preg_replace("/\\\'/", "'", $iepg[$key]));
		if ($key == 'program-title'
			|| $key == 'start'
			|| $key == 'end'
			|| $key == 'iepg-url') {
			$iepgtext .= sprintf("%s-ORIGINAL: %s\n", $key, $iepg[$key]);
		}
	}

	$minutes	= programTime($iepg);
	$etime	= programStart($iepg) + $minutes * 60;
	$gonogo	= 'has been';

	// Make sure we have at least 1 minute to record a one-time show
	if (!@$iepg['every'] && $etime < time() + 60) {
		// Too short or past one-time recording is an error
		$alerttext	.= '*** ERROR ***\nProgram has ended or is about to end.\n';
		$gonogo	= 'was not';
	} else {
		$shm	= preg_split('/:/', $iepg['start']);
		if (@$iepg['every']) {
			$filename	= sprintf("%s-%02d%02d-%03d-%02d%02d-%s",
							$DOWMAP[$iepg['every']],
							$shm[0], $shm[1],
							$iepg['channel'],
							$minutes/60, $minutes%60,
							$signature);
		} else {
			$filename	= sprintf("%s%s%s%02d%02d-%03d-%02d%02d-%s",
							$iepg['year'], $iepg['month'], $iepg['date'],
							$shm[0], $shm[1],
							$iepg['channel'],
							$minutes/60, $minutes%60,
							$signature);
			// Make sure there's no contradictory leftover
			@unlink("$VIDEOROOTDIR/scheduled/$filename.skip");
		}

		if (!($fh = fopen("$VIDEOROOTDIR/regular/$filename.iepg", 'w'))) {
			exit(1);
		}
		fwrite($fh, $iepgtext);
		fclose($fh);
	}

	$DOW	= array('Mon' => 'Monday', 'Tue' => 'Tuesday', 'Wed' => 'Wednesday',
					'Thu' => 'Thursday','Fri' => 'Friday',  'Sat' => 'Saturday',
					'Sun' => 'Sunday', 'M-F' => 'Monday thru Friday');
	$alerttext	.= sprintf('Recording of \"%s\"\n%s scheduled ',
							$iepg['program-title'], $gonogo);
	switch($program) {
	case 'once':
		$alerttext .= sprintf("for this epsode.");
		break;
	default:
		$alerttext .= sprintf("for every %s.", $DOW[$program]);
		break;
	}
}

//
// Project regular schedule and dispatch a recording process.
// Run update-schedule in background as it may run for a few seconds or more.
// The order of update-schedule options is important. Scheduling first, then
// recording!
// *** If the command line is ever changed, /etc/sudoers must be updated ***
//
if (@$filename) {
	system("$SUDO $RCMDPATH/update-schedule -s $filename -r >/dev/null 2>&1 &");
}

$now	= time();

printDocHeader();
echo <<<END
<HTML>
<HEAD>
<meta http-equiv="Content-Type" content="text/html;charset=shift_jis" />
</HEAD>
<BODY BGCOLOR='$bgcolor'>
<SCRIPT LANGUAGE=JavaScript>
<!--
var alerttext	= '$alerttext';
var choice	= '$choice';

window.open('6ga-panel.php?tab=$thistab&_time_=$now', '_6ga_panel');
if (choice == 'schedule') {
	alert(alerttext);
	window.open('6ga-list.php?tab=$thistab&_time=$now', '_6ga_list');
}
//-->
</SCRIPT>
</BODY>
</HTML>
END;
?>
